' Example 3b:  Verify a secret on a SmartDongle.
'
' MicroWorks, Inc.
' 2808 North Cole Road
' Boise, ID 83704
'
' http://www.smartdongle.com

Imports System
Imports System.Security.Cryptography
Imports System.Text
Imports System.IO

Module Example3b

    Sub Main()

        Dim dongle As New SmartDongle()
        Dim dnglError As SmartDongle.SmartDnglError
        Dim encoding As System.Text.ASCIIEncoding = New System.Text.ASCIIEncoding()


        '  Your keys here
        Dim P1 As Int64 = &HEC6CC589AEFD1E75
        Dim P2 As Int64 = &HFCEC0A6A82747B3F


        ' Your secret data, make a copy in a byte array
        Dim your_secret As String = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc"
        Dim plainText() As Byte = encoding.GetBytes(your_secret)


        ' Padding is zeroes and CipherMode has been set to
        ' Electronic Codebook (ECB) to match the algorithm in the C examples.
        '
        ' The AES key and user secret below are for example only, the user
        ' should create a new key and secret for their application.
        '
        '  A System.Security.Cryptography.CryptographicException will
        '  occur if illegal AES key length or AES key size are given below.

        Dim aes_key As String = "7obdgEn2'TH>]SE|"
        Dim aes_KeySize As Integer = 128
        Dim aes_BlockSize As Integer = 128
        Dim RijndaelAlg As Rijndael
        Dim rgbKey() As Byte
        Dim rgbIV() As Byte
        Dim decrypt As ICryptoTransform
        Dim result() As Byte

Try
        RijndaelAlg = Rijndael.Create()
        RijndaelAlg.KeySize = aes_KeySize
        RijndaelAlg.BlockSize = aes_BlockSize
        RijndaelAlg.Mode = CipherMode.ECB
        RijndaelAlg.Padding = PaddingMode.Zeros

        ' Initial vector (rgbIV) is the same as the key as in the C example code
        rgbKey = encoding.GetBytes(aes_key)
        rgbIV = rgbKey

        decrypt = RijndaelAlg.CreateDecryptor(rgbKey, rgbIV)

Catch ex As System.Security.Cryptography.CryptographicException
        MsgBox("Failed to initialized Rijndael Decryption Objects")

        ' handle error
        Return
End Try


        '
        ' Cipher length read from SmartDongle is a multiple of AES blocksize.
        ' Note that AES blocksize is in bits.
        '
        Dim cipherBlocks As Integer = CType(your_secret.Length / (aes_BlockSize / 8), Integer)
        If (your_secret.Length Mod (aes_BlockSize / 8) > 0) Then
            cipherBlocks = cipherBlocks + 1
        End If


        '
        ' Read the encrypted secret from the SmartDongle,
        '
        Dim cipherText(CType((cipherBlocks * (aes_BlockSize / 8)) - 1, Integer)) As Byte
        Dim address As Int16 = 0 ' User memory address. Byte addressable from 0 to 32431
        dnglError = dongle.Read(P1, P2, address, cipherText, cipherText.Length)

        If dnglError <> SmartDongle.SmartDnglError.UskOk Then

            System.Console.WriteLine("Failed: Reading Secret back from SmartDongle")
            System.Console.WriteLine(SmartDongle.GetErrorString(dnglError))

            ' handle error
            Return
        End If

Try
        '
        ' Decrypt
        '
        result = decrypt.TransformFinalBlock(cipherText, 0, cipherText.Length)

Catch ex As System.Security.Cryptography.CryptographicException

        MsgBox("Failure of Rijndael decryption occurred")

        ' handle error
        Return
End Try

        ' Compare to original secret, using orignal secret length since the
        ' result will be a multiple of block length and padded with zeroes
        ' The data comparison here is just an exercise, the developer can do
        ' any number of things with the decrypted user data here.
        Dim i As Integer
        For i = 1 To plainText.Length - 1
            If (result(i) <> plainText(i)) Then
                Exit For
            End If
        Next

        If i = plainText.Length Then
            System.Console.WriteLine("Success: Found Valid Secret")
        Else
            System.Console.WriteLine("Failed: Invalid Secret")
        End If

        System.Console.WriteLine("")
        System.Console.WriteLine("Press <Enter> to Exit")
        System.Console.ReadLine()
    End Sub

End Module
